"""Automated embedding of resources in Python Packages/Modules

ResourcePackage provides the Python programmer with a
mechanism for automatically maintaining a collection of
resources embedded in Python source-code modules.

The basic approach taken is to embed one resource in
each python module of a package explicitly intended
to be a resource-only package.

See the doc sub-directory for documentation
"""
__version__ = (1,0,0)
__author__ = "Michael Colin Fletcher"
__license__ = "BSD-Style, see license.txt for details"

