# -*- coding: ISO-8859-1 -*-
"""Resource _cvsignore (from file .cvsignore)"""
# written by resourcepackage: (1, 0, 0)
source = '.cvsignore'
package = 'resourcepackage.testwx'
data = "*.pyc\015\012*.pyo\015\012*.max\015\012*.db\015\012*.bat\015\012*.cvsignore\015\012*.cache\015\012MANI\
FEST\015\012\015\012"
### end
