"""Script to automatically generate OpenGLContext documentation"""
from OpenGLContext.pydoc import pydoc2

if __name__ == "__main__":
	excludes = [
		"wxPython.wx",
		"Numeric",
		"_tkinter",
		"Tkinter",
		"math",
	]
	stops = [
##		"resourcepackage.scanning__init__",
	]

	modules = [
		"resourcepackage",
	]	
	pydoc2.PackageDocumentationGenerator(
		baseModules = modules,
		destinationDirectory = "../doc/pydoc",
		exclusions = excludes,
		recursionStops = stops,
	).process ()
